use strict;
use warnings;
use utf8;
use Test::More;
use File::Spec;
use open IO => ':utf8', ':std';
use Data::Dumper;
$Data::Dumper::Sortkeys = 1;

use lib '.';
use t::Runner;

sub greple {
    Runner->new('greple', @_)->run;
}

1;
